# encoding: utf-8
require "logstash/outputs/base"
require "logstash/namespace"
require "baidubce/services/bos/bos_client"
require "rufus-scheduler"
include Baidubce

# Upload the log string into specified bos
class LogStash::Outputs::Bos < LogStash::Outputs::Base
  config_name "bos"

  # The log string to be stored in bos
  # config :log, :validate => :string, :required => true

  # Destination bucket name
  config :bucket, :validate => :string, :required => true

  # Object Name
  config :object, :validate => :string, :required => true

  # Path of the file to upload
  config :file_path, :validate => :string, :required => true

  # Endpoint of BOS
  config :endpoint, :validate => :string, :required => true

  # Access Key of BOS
  config :ak, :validate => :string, :required => true

  # Secret Key of BOS
  config :sk, :validate => :string, :required => true

  public
  def register
    credentials = Auth::BceCredentials.new(@ak, @sk)
    conf = BceClientConfiguration.new(credentials, @endpoint)
    @client = Services::BosClient.new(conf)
    @@scheduledTasks = {}
    # 时区设置
    ENV['TZ'] = 'Asia/Shanghai'
  end # def register

  public
  def receive(event)
    if @@scheduledTasks["#{@bucket} - #{@object}"].nil?
      # 上传任务未计划，增加新的任务
      scheduler = Rufus::Scheduler.new
      scheduler.at "#{Date.today.next_day.strftime('%Y-%m-%d')} 00:10" do
        # 在第二天0点10分上传前一天的csv文件
        begin
          @client.put_object_from_file(@bucket, @object, @file_path)
          @@scheduledTasks["#{@bucket} - #{@object}"] = nil
        rescue BceServerException => e
          puts "Error: #{e}"
        end
      end
      @@scheduledTasks["#{@bucket} - #{@object}"] = scheduler
      scheduler.join
    end
  end # def receive

end # class LogStash::Outputs::Bos